// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

#include <iostream>

#include <amdis/AMDiS.hpp>
#include <amdis/AdaptInstationary.hpp>
#include <amdis/ProblemInstat.hpp>
#include <amdis/ProblemStat.hpp>
#include <amdis/Operators.hpp>
#include <amdis/GridFunctions.hpp>
#include <amdis/common/Literals.hpp>

using namespace AMDiS;

int main(int argc, char** argv)
{
  AMDiS::init(argc, argv);

  using Grid = Dune::YaspGrid<AMDIS_DIM>;
  auto gridPtr = MeshCreator<Grid>::create("mesh");
  auto basis = makeBasis(gridPtr->leafGridView(), lagrange<2>());

  auto prob = makeProblemStat("heat", *gridPtr, basis);
  prob.initialize(INIT_ALL);

  auto probInstat = makeProblemInstat("heat", prob);
  probInstat.initialize(INIT_UH_OLD);

  AdaptInfo adaptInfo("adapt");

  double* invTau = probInstat.getInvTau();

  auto _ = Dune::TypeTree::hybridTreePath();

  auto opTimeLhs = makeOperator(tag::test_trial{}, std::ref(*invTau));
  prob.addMatrixOperator(opTimeLhs, _, _);

  auto opL = makeOperator(tag::gradtest_gradtrial{}, 1.0);
  prob.addMatrixOperator(opL, _, _);

  auto opTimeRhs = makeOperator(tag::test{},
    invokeAtQP([invTau](double u) { return u * (*invTau); }, prob.getSolution()), 2);
  prob.addVectorOperator(opTimeRhs, _);

  auto opForce = makeOperator(tag::test{}, [](auto const& x) { return -1.0; }, 0);
  prob.addVectorOperator(opForce, _);


  // set boundary condition
  auto predicate = [](auto const& p){ return p[0] < 1.e-8 || p[1] < 1.e-8; };
  auto dbcValues = [](auto const& p){ return 0.0; };
  prob.addDirichletBC(predicate, _, _, dbcValues);

  AdaptInstationary adapt("adapt", prob, adaptInfo, probInstat, adaptInfo);
  adapt.adapt();

  AMDiS::finalize();
  return 0;
}
